from pydantic import BaseModel, ConfigDict, StringConstraints, field_validator
from typing_extensions import Annotated, Optional

from corbado_python_sdk.utils import validators


class Config(BaseModel):
    """
    Configuration class for setting up project parameters.

    This class uses Pydantic's `BaseModel` to validate configuration parameters.
    Field assignments are validated by default to ensure that any new assignments
    adhere to the defined types and constraints. To disable this validation, set
    `set_assignment_validation(False)`.

    Attributes:
        project_id (str): The unique identifier for the project.
        api_secret (str): The secret key used to authenticate API requests.
        backend_api (str): The base URL for the backend API. Defaults to "https://backendapi.cloud.corbado.io/v2".
        short_session_cookie_name (str): The name of the cookie for short session management. Defaults to "cbo_short_session".
    """

    # Make sure that field assignments are also validated, use "set_assignment_validation(False)"
    # to be able to use invalid assignments
    model_config = ConfigDict(validate_assignment=True)

    # Fields
    project_id: str
    api_secret: str

    backend_api: str = "https://backendapi.cloud.corbado.io/v2"
    short_session_cookie_name: str = "cbo_short_session"

    _issuer: Optional[Annotated[str, StringConstraints(strip_whitespace=True, min_length=1)]] = None
    _frontend_api: Optional[str] = None

    @field_validator("backend_api")
    @classmethod
    def validate_backend_api(cls, backend_api: str) -> str:
        """Validate the backend API URL and ensure it ends with '/v2'.

        Args:
            backend_api (str): Backend API URL to validate.

        Raises:
            ValueError: _description_

        Returns:
            str: Validated backend API URL ending with '/v2'.
        """
        if not validators.url_validator(backend_api):
            raise ValueError(f'Invalid URL "{backend_api}" provided for backend API.')

        # Append '/v2' if not already present
        if not backend_api.endswith("/v2"):
            return backend_api.rstrip("/") + "/v2"

        return backend_api

    @field_validator("project_id")
    @classmethod
    def project_id_validator(cls, project_id: str) -> str:
        """Field validator for project_ID.

        Args:
            project_id (str): project ID

        Raises:
            ValueError: Raises ValueError if project_id does not start with "pro-"

        Returns:
            str: validated project ID
        """
        if not project_id.startswith("pro-"):
            raise ValueError(f'Invalid project ID "{project_id}" given, needs to start with "pro-"')
        return project_id

    @field_validator("api_secret")
    @classmethod
    def api_secret_must_start_with_corbado1(cls, api_secret: str) -> str:
        """Field validator for api_secret.

        Args:
            api_secret (str): API secret

        Raises:
            ValueError: Raises ValueError if api_secret does not start with "corbado1_"

        Returns:
            str: validated API secret
        """
        if not api_secret.startswith("corbado1_"):
            raise ValueError(f'Invalid API Secret "{api_secret}" given, needs to start with "corbado1_"')
        return api_secret

    # --------- Properties ----------#
    @property
    def issuer(self) -> str:
        """Get issuer. By default issuer = frontend_api, but can be overridden.

        Returns:
            str: issuer.
        """
        if not self._issuer:
            self._issuer = self.frontend_api
        return self._issuer

    @issuer.setter
    def issuer(self, issuer: str) -> None:
        """Set issuer.

        Args:
            issuer (str): issuer to set.
        """
        self._issuer = issuer

    @property
    def frontend_api(self) -> str:
        """Get Frontend API.

        Returns:
            str: Frontend API
        """
        if not self._frontend_api:
            self._frontend_api = "https://" + self.project_id + ".frontendapi.corbado.io"
        return self._frontend_api

    @frontend_api.setter
    def frontend_api(self, frontend_api: str) -> None:
        """Set Frontend API. Use it to override default value.

        Args:
            frontend_api (str): Frontend API to set.
        """
        self._frontend_api = validators.url_validator(url=frontend_api)  # validate url

    # ------- Internal --------------#
    def set_assignment_validation(self, validate: bool) -> None:
        """Only use it if you know what you do. Sets assignment validation.

        Args:
            validate (bool): Enable/disable validation
        """
        self.model_config["validate_assignment"] = validate



from .interface import IdentifierInterface as IdentifierInterface
from .interface import SessionInterface as SessionInterface
from .interface import UserInterface as UserInterface

__all__ = [
    "IdentifierInterface",
    "UserInterface",
    "SessionInterface",
]



import jwt
from jwt import decode
from jwt.exceptions import PyJWTError
from jwt.jwks_client import PyJWKClient
from pydantic import BaseModel, ConfigDict, StringConstraints
from typing_extensions import Annotated

from corbado_python_sdk.entities.session_validation_result import (
    SessionValidationResult,
)
from corbado_python_sdk.services.interface import SessionInterface

DEFAULT_SHORT_SESSION_LENGTH = 300


class SessionService(SessionInterface, BaseModel):
    """
    Implementation of SessionInterface.

    This class provides functionality for managing sessions, including validation and retrieval of user information
    from short-term session tokens.

    Attributes:
        model_config (ConfigDict): Configuration dictionary for the model.
        short_session_cookie_name (str): Name of the short session cookie.
        issuer (str): Issuer of the session tokens.
        jwks_uri (str): URI of the JSON Web Key Set (JWKS) endpoint.
        last_short_session_validation_result (str): Result of the last short session validation.
        short_session_length (int): Length of short session in seconds. Default = 300
        _jwk_client (PyJWKClient): JSON Web Key (JWK) client for handling JWKS.
        cache_keys (bool): Flag to cache keys. Default = False.
        cache_jwk_set (bool): Flag to cache jwk_sets. Default = True.


    """

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )
    short_session_cookie_name: Annotated[str, StringConstraints(strip_whitespace=True, min_length=1)]
    issuer: Annotated[str, StringConstraints(strip_whitespace=True, min_length=1)]
    jwks_uri: Annotated[str, StringConstraints(strip_whitespace=True, min_length=1)]
    last_short_session_validation_result: str = ""
    short_session_length: int = DEFAULT_SHORT_SESSION_LENGTH
    cache_keys: bool = False
    cache_jwk_set: bool = True
    _jwk_client: PyJWKClient

    def __init__(self, **kwargs) -> None:  # type: ignore
        """
        Initialize a new instance of the SessionService class.

        Args:
            **kwargs: Additional keyword arguments to initialize the SessionService.
                These keyword arguments should include values for the attributes defined in the class,
                such as 'short_session_cookie_name', 'issuer', 'jwks_uri', 'last_short_session_validation_result',
                'cache_keys',cache_jwk_set and 'short_session_length'.

        Raises:
            Any errors raised during the initialization process.

        """
        super().__init__(**kwargs)
        self._jwk_client = PyJWKClient(
            uri=self.jwks_uri,
            cache_keys=self.cache_keys,
            lifespan=self.short_session_length,
            cache_jwk_set=self.cache_jwk_set,
        )

    def get_and_validate_short_session_value(self, short_session: str) -> SessionValidationResult:
        """Validate the given short-term session (represented as JWT) value.

        Args:
            short_session (str): jwt

        Returns:
            SessionValidationResult: SessionValidationResult with authenticated=True on success,
            otherwise with authenticated=False
        """
        if not short_session:
            return SessionValidationResult(authenticated=False)
        try:
            # retrieve signing key
            signing_key: jwt.PyJWK = self._jwk_client.get_signing_key_from_jwt(token=short_session)
            # decode short session (jwt) with signing key
            payload = decode(jwt=short_session, key=signing_key.key, algorithms=["RS256"])

            # extract information from decoded payload
            sub = payload.get("sub")
            full_name = payload.get("name")

            # check issuer
            if payload.get("iss") and payload["iss"] != self.issuer:
                self.set_issuer_mismatch_error(issuer=payload["iss"])
                # return unauthenticated user on issuer mismatch
                return SessionValidationResult(authenticated=False)
            return SessionValidationResult(authenticated=True, user_id=sub, full_name=full_name)
        except PyJWTError as error:
            # return unauthenticated user on error
            self.set_validation_error(error)
            return SessionValidationResult(authenticated=False)

    def get_current_user(self, short_session: str) -> SessionValidationResult:
        """Return current user for the short session.

        Args:
            short_session (str): Short session.

        Returns:
            SessionValidationResult:  SessionValidationResult with authenticated=True on success, otherwise with
                authenticated=False.
        """
        if not short_session:
            return SessionValidationResult(authenticated=False)

        user: SessionValidationResult = self.get_and_validate_short_session_value(short_session)
        return user

    def set_issuer_mismatch_error(self, issuer: str) -> None:
        """Set issuer mismatch error.

        Args:
            issuer (str): issuer.
        """
        self.last_short_session_validation_result = f"Mismatch in issuer (configured: {self.issuer}, JWT: {issuer})"

    def set_validation_error(self, error: Exception) -> None:
        """Set validation error.

        Args:
            error (Exception): Exception occurred.
        """
        self.last_short_session_validation_result = f"JWT validation failed: {error}"



from .identifier_service import IdentifierService
from .session_service import SessionService
from .user_service import UserService

__all__ = [
    "IdentifierService",
    "UserService",
    "SessionService",
]



from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Annotated, Optional

from corbado_python_sdk.entities import UserEntity
from corbado_python_sdk.exceptions.server_exception import ServerException
from corbado_python_sdk.generated.api import UsersApi
from corbado_python_sdk.generated.exceptions import ApiException
from corbado_python_sdk.generated.models import UserCreateReq
from corbado_python_sdk.generated.models.user import User
from corbado_python_sdk.generated.models.user_status import UserStatus
from corbado_python_sdk.services.interface import UserInterface


class UserService(
    BaseModel,
    UserInterface,
):
    """Service for managing users."""

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )
    client: UsersApi

    def create_from_request(self, request: UserCreateReq) -> UserEntity:
        """Create a user from UserCreateReq.

        Args:
            request (UserCreateReq):  User create request

        Raises:
            ServerException: If any server side error occurs.

        Returns:
            UserEntity: _description_
        """
        try:
            user: User = self.client.user_create(user_create_req=request)
        except ApiException as e:
            raise ServerException(e)

        return UserEntity.from_user(user)

    def create(
        self,
        status: UserStatus,
        full_name: Optional[StrictStr] = None,
        explicit_webauthn_id: Optional[StrictStr] = None,
    ) -> UserEntity:
        """Create a user.

        Args:
            status (UserStatus): User status.
            full_name (Optional[StrictStr], optional): Full name. Defaults to None.
            explicit_webauthn_id (Optional[StrictStr], optional): explicit_webauthn_id. Defaults to None.

        Raises:
            ServerException:  If any server side error occurs.

        Returns:
            UserEntity: UserEntity.
        """
        request = UserCreateReq(status=status, fullName=full_name, explicitWebauthnID=explicit_webauthn_id)
        try:
            user: User = self.client.user_create(user_create_req=request)
        except ApiException as e:
            raise ServerException(e)

        return UserEntity.from_user(user)

    def get(self, user_id: Annotated[StrictStr, Field(description="ID of user")]) -> UserEntity:
        """Retrieve user from userId.

        Args:
            user_id (Annotated[StrictStr, Field, optional): UserId.)].

        Raises:
            ServerException:  If any server side error occurs.

        Returns:
            UserEntity: UserEntity.
        """
        try:
            user: User = self.client.user_get(user_id=user_id)
        except ApiException as e:
            raise ServerException(e)

        return UserEntity.from_user(user)

    def delete(self, user_id: str) -> None:
        """Delete user. Does not return anything. Throw if any error occurs (Like user not exists).

        Args:
            user_id (str): UserId.

        Raises:
            ServerException: If any server side error occurs.
        """
        try:
            self.client.user_delete(user_id=user_id)
        except ApiException as e:
            raise ServerException(e)



# coding: utf-8

"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List
from typing import Optional, Set
from typing_extensions import Self

class Credential(BaseModel):
    """
    Credential
    """ # noqa: E501
    id: StrictStr
    credential_id: StrictStr = Field(alias="credentialID")
    attestation_type: StrictStr = Field(alias="attestationType")
    transport: List[StrictStr]
    backup_eligible: StrictBool = Field(alias="backupEligible")
    backup_state: StrictBool = Field(alias="backupState")
    authenticator_aaguid: StrictStr = Field(alias="authenticatorAAGUID")
    source_os: StrictStr = Field(alias="sourceOS")
    source_browser: StrictStr = Field(alias="sourceBrowser")
    last_used: StrictStr = Field(description="Timestamp of when the passkey was last used in yyyy-MM-dd'T'HH:mm:ss format", alias="lastUsed")
    created: StrictStr = Field(description="Timestamp of when the entity was created in yyyy-MM-dd'T'HH:mm:ss format")
    status: StrictStr = Field(description="Status")
    __properties: ClassVar[List[str]] = ["id", "credentialID", "attestationType", "transport", "backupEligible", "backupState", "authenticatorAAGUID", "sourceOS", "sourceBrowser", "lastUsed", "created", "status"]

    @field_validator('transport')
    def transport_validate_enum(cls, value):
        """Validates the enum"""
        for i in value:
            if i not in set(['usb', 'nfc', 'ble', 'internal', 'hybrid', 'smart-card']):
                raise ValueError("each list item must be one of ('usb', 'nfc', 'ble', 'internal', 'hybrid', 'smart-card')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['pending', 'active']):
            raise ValueError("must be one of enum values ('pending', 'active')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Credential from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Credential from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "credentialID": obj.get("credentialID"),
            "attestationType": obj.get("attestationType"),
            "transport": obj.get("transport"),
            "backupEligible": obj.get("backupEligible"),
            "backupState": obj.get("backupState"),
            "authenticatorAAGUID": obj.get("authenticatorAAGUID"),
            "sourceOS": obj.get("sourceOS"),
            "sourceBrowser": obj.get("sourceBrowser"),
            "lastUsed": obj.get("lastUsed"),
            "created": obj.get("created"),
            "status": obj.get("status")
        })
        return _obj





# coding: utf-8

# flake8: noqa
"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from corbado_python_sdk.generated.models.app_type import AppType
from corbado_python_sdk.generated.models.auth_event import AuthEvent
from corbado_python_sdk.generated.models.auth_event_create_req import AuthEventCreateReq
from corbado_python_sdk.generated.models.auth_event_method import AuthEventMethod
from corbado_python_sdk.generated.models.auth_event_status import AuthEventStatus
from corbado_python_sdk.generated.models.auth_event_type import AuthEventType
from corbado_python_sdk.generated.models.challenge import Challenge
from corbado_python_sdk.generated.models.challenge_create_req import ChallengeCreateReq
from corbado_python_sdk.generated.models.challenge_status import ChallengeStatus
from corbado_python_sdk.generated.models.challenge_type import ChallengeType
from corbado_python_sdk.generated.models.challenge_update_req import ChallengeUpdateReq
from corbado_python_sdk.generated.models.client_information import ClientInformation
from corbado_python_sdk.generated.models.connect_token import ConnectToken
from corbado_python_sdk.generated.models.connect_token_create_req import ConnectTokenCreateReq
from corbado_python_sdk.generated.models.connect_token_data import ConnectTokenData
from corbado_python_sdk.generated.models.connect_token_data_passkey_append import ConnectTokenDataPasskeyAppend
from corbado_python_sdk.generated.models.connect_token_data_passkey_delete import ConnectTokenDataPasskeyDelete
from corbado_python_sdk.generated.models.connect_token_data_passkey_list import ConnectTokenDataPasskeyList
from corbado_python_sdk.generated.models.connect_token_list import ConnectTokenList
from corbado_python_sdk.generated.models.connect_token_status import ConnectTokenStatus
from corbado_python_sdk.generated.models.connect_token_type import ConnectTokenType
from corbado_python_sdk.generated.models.connect_token_update_req import ConnectTokenUpdateReq
from corbado_python_sdk.generated.models.credential import Credential
from corbado_python_sdk.generated.models.credential_list import CredentialList
from corbado_python_sdk.generated.models.cross_device_authentication_strategy import CrossDeviceAuthenticationStrategy
from corbado_python_sdk.generated.models.decision_tag import DecisionTag
from corbado_python_sdk.generated.models.detection_tag import DetectionTag
from corbado_python_sdk.generated.models.error_rsp import ErrorRsp
from corbado_python_sdk.generated.models.error_rsp_all_of_error import ErrorRspAllOfError
from corbado_python_sdk.generated.models.error_rsp_all_of_error_validation import ErrorRspAllOfErrorValidation
from corbado_python_sdk.generated.models.generic_rsp import GenericRsp
from corbado_python_sdk.generated.models.identifier import Identifier
from corbado_python_sdk.generated.models.identifier_create_req import IdentifierCreateReq
from corbado_python_sdk.generated.models.identifier_list import IdentifierList
from corbado_python_sdk.generated.models.identifier_status import IdentifierStatus
from corbado_python_sdk.generated.models.identifier_type import IdentifierType
from corbado_python_sdk.generated.models.identifier_update_req import IdentifierUpdateReq
from corbado_python_sdk.generated.models.java_script_high_entropy import JavaScriptHighEntropy
from corbado_python_sdk.generated.models.long_session import LongSession
from corbado_python_sdk.generated.models.long_session_create_req import LongSessionCreateReq
from corbado_python_sdk.generated.models.long_session_status import LongSessionStatus
from corbado_python_sdk.generated.models.long_session_update_req import LongSessionUpdateReq
from corbado_python_sdk.generated.models.paging import Paging
from corbado_python_sdk.generated.models.passkey_append_finish_req import PasskeyAppendFinishReq
from corbado_python_sdk.generated.models.passkey_append_finish_rsp import PasskeyAppendFinishRsp
from corbado_python_sdk.generated.models.passkey_append_start_req import PasskeyAppendStartReq
from corbado_python_sdk.generated.models.passkey_append_start_rsp import PasskeyAppendStartRsp
from corbado_python_sdk.generated.models.passkey_challenge import PasskeyChallenge
from corbado_python_sdk.generated.models.passkey_challenge_list import PasskeyChallengeList
from corbado_python_sdk.generated.models.passkey_challenge_status import PasskeyChallengeStatus
from corbado_python_sdk.generated.models.passkey_challenge_type import PasskeyChallengeType
from corbado_python_sdk.generated.models.passkey_challenge_update_req import PasskeyChallengeUpdateReq
from corbado_python_sdk.generated.models.passkey_data import PasskeyData
from corbado_python_sdk.generated.models.passkey_event import PasskeyEvent
from corbado_python_sdk.generated.models.passkey_event_create_req import PasskeyEventCreateReq
from corbado_python_sdk.generated.models.passkey_event_list import PasskeyEventList
from corbado_python_sdk.generated.models.passkey_event_type import PasskeyEventType
from corbado_python_sdk.generated.models.passkey_intel_flags import PasskeyIntelFlags
from corbado_python_sdk.generated.models.passkey_login_finish_req import PasskeyLoginFinishReq
from corbado_python_sdk.generated.models.passkey_login_finish_rsp import PasskeyLoginFinishRsp
from corbado_python_sdk.generated.models.passkey_login_start_req import PasskeyLoginStartReq
from corbado_python_sdk.generated.models.passkey_login_start_rsp import PasskeyLoginStartRsp
from corbado_python_sdk.generated.models.passkey_mediation_finish_req import PasskeyMediationFinishReq
from corbado_python_sdk.generated.models.passkey_mediation_finish_rsp import PasskeyMediationFinishRsp
from corbado_python_sdk.generated.models.passkey_mediation_start_req import PasskeyMediationStartReq
from corbado_python_sdk.generated.models.passkey_mediation_start_rsp import PasskeyMediationStartRsp
from corbado_python_sdk.generated.models.project_config_update_cname_req import ProjectConfigUpdateCnameReq
from corbado_python_sdk.generated.models.request_data import RequestData
from corbado_python_sdk.generated.models.short_session import ShortSession
from corbado_python_sdk.generated.models.short_session_create_req import ShortSessionCreateReq
from corbado_python_sdk.generated.models.social_account import SocialAccount
from corbado_python_sdk.generated.models.social_account_create_req import SocialAccountCreateReq
from corbado_python_sdk.generated.models.social_account_list import SocialAccountList
from corbado_python_sdk.generated.models.social_provider_type import SocialProviderType
from corbado_python_sdk.generated.models.user import User
from corbado_python_sdk.generated.models.user_create_req import UserCreateReq
from corbado_python_sdk.generated.models.user_status import UserStatus
from corbado_python_sdk.generated.models.user_update_req import UserUpdateReq



# coding: utf-8

"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AuthEventMethod(str, Enum):
    """
    AuthEventMethod
    """

    """
    allowed enum values
    """
    PASSWORD = 'password'
    EMAIL_OTP = 'email_otp'
    EMAIL_LINK = 'email_link'
    PHONE_OTP = 'phone_otp'
    PASSKEY = 'passkey'
    SOCIAL_GITHUB = 'social_github'
    SOCIAL_GOOGLE = 'social_google'
    SOCIAL_MICROSOFT = 'social_microsoft'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AuthEventMethod from a JSON string"""
        return cls(json.loads(json_str))





from abc import ABC, abstractmethod

from pydantic import Field, StrictStr
from typing_extensions import Annotated, Optional

from corbado_python_sdk.entities import UserEntity
from corbado_python_sdk.generated.models import UserCreateReq
from corbado_python_sdk.generated.models.user_status import UserStatus


class UserInterface(ABC):
    """Interface containing functions for interaction with users."""

    @abstractmethod
    def create_from_request(self, request: UserCreateReq) -> UserEntity:
        """Create a user from UserCreateReq.

        Args:
            request (UserCreateReq):  User create request

        Raises:
            ServerException: If any server side error occurs.

        Returns:
            UserEntity: _description_
        """

    @abstractmethod
    def create(
        self,
        status: UserStatus,
        full_name: Optional[StrictStr] = None,
        explicit_webauthn_id: Optional[StrictStr] = None,
    ) -> UserEntity:
        """Create a user.

        Args:
            status (UserStatus): User status.
            full_name (Optional[StrictStr], optional): Full name. Defaults to None.
            explicit_webauthn_id (Optional[StrictStr], optional): explicit_webauthn_id. Defaults to None.

        Raises:
            ServerException:  If any server side error occurs.

        Returns:
            UserEntity: UserEntity.
        """

    @abstractmethod
    def get(self, user_id: Annotated[StrictStr, Field(description="ID of user")]) -> UserEntity:
        """Retrieve user from userId.

        Args:
            user_id (Annotated[StrictStr, Field, optional): UserId.)].

        Raises:
            ServerException:  If any server side error occurs.

        Returns:
            UserEntity: UserEntity.
        """

    @abstractmethod
    def delete(self, user_id: str):
        """Delete user. Does not return anything. Throw if any error occurs (Like user not exists).

        Args:
            user_id (str): UserId.

        Raises:
            ServerException: If any server side error occurs.
        """



# flake8: noqa

# import apis into api package
from corbado_python_sdk.generated.api.auth_events_api import AuthEventsApi
from corbado_python_sdk.generated.api.challenges_api import ChallengesApi
from corbado_python_sdk.generated.api.connect_tokens_api import ConnectTokensApi
from corbado_python_sdk.generated.api.identifiers_api import IdentifiersApi
from corbado_python_sdk.generated.api.passkey_challenges_api import PasskeyChallengesApi
from corbado_python_sdk.generated.api.passkey_events_api import PasskeyEventsApi
from corbado_python_sdk.generated.api.passkeys_api import PasskeysApi
from corbado_python_sdk.generated.api.project_config_api import ProjectConfigApi
from corbado_python_sdk.generated.api.sessions_api import SessionsApi
from corbado_python_sdk.generated.api.users_api import UsersApi




from abc import ABC, abstractmethod

from corbado_python_sdk.entities.session_validation_result import (
    SessionValidationResult,
)


class SessionInterface(ABC):
    """SessionInterface for managing sessions."""

    @abstractmethod
    def get_and_validate_short_session_value(self, short_session: str) -> SessionValidationResult:
        """Validate  the given short-term session value.

        Args:
            short_session (str): The value (JWT) to validate.

        Returns:
            Optional[Decoded]: Returns Decoded object on success, otherwise None.
        """
        pass

    @abstractmethod
    def get_current_user(self, short_session: str) -> SessionValidationResult:
        """Return current user for the short session.

        Args:
            short_session (str): Short session.

        Returns:
            UserEntity:  UserEntity with authenticated=True on success, otherwise with authenticated=False.
        """
        pass



import logging
import unittest

from typing_extensions import List

from corbado_python_sdk import CorbadoSDK, Identifier, IdentifierStatus, IdentifierType
from corbado_python_sdk.exceptions import ServerException
from corbado_python_sdk.generated.models.identifier_list import IdentifierList
from corbado_python_sdk.services.interface import IdentifierInterface
from tests.utils import TestUtils

CANNOT_BE_BLANK = "cannot be blank"
logging.basicConfig(level=logging.DEBUG)
urllib3_logger = logging.getLogger("urllib3")
urllib3_logger.setLevel(logging.DEBUG)


class TestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        """Set up test class with necessary data."""
        cls.sdk: CorbadoSDK = TestUtils.instantiate_sdk()
        cls.fixture: IdentifierInterface = cls.sdk.identifiers

        cls.TEST_USER_ID = TestUtils.create_user().user_id
        cls.TEST_USER_EMAIL: str = TestUtils.create_random_test_email()
        cls.TEST_USER_PHONE: str = TestUtils.create_random_test_phone_number()

        cls.TEST_USER_EMAIL_IDENTIFIER: Identifier = cls.fixture.create(
            user_id=cls.TEST_USER_ID,
            identifier_type=IdentifierType.EMAIL,
            identifier_value=cls.TEST_USER_EMAIL,
            status=IdentifierStatus.PRIMARY,
        )

        cls.fixture.create(
            user_id=cls.TEST_USER_ID,
            identifier_type=IdentifierType.PHONE,
            identifier_value=cls.TEST_USER_PHONE,
            status=IdentifierStatus.PRIMARY,
        )


class TestIdentifierService(TestBase):
    """Test cases for IdentifierService."""

    def test_check_existing_email_is_present_expect_success(self):
        """Test getting email and non-matching phone identifiers."""
        ret: IdentifierList = self.fixture.list_identifiers(
            identifier_type=IdentifierType.EMAIL, identifier_value=self.TEST_USER_EMAIL
        )
        self.assertEqual(first=1, second=ret.paging.total_items)  # assert only one TEST_USER_EMAIL exists in list

        self.assertFalse(
            self.fixture.exists_by_value_and_type(
                value=self.TEST_USER_EMAIL,
                identifier_type=IdentifierType.PHONE,  # assert Identifier Type works correctly
            )
        )
        self.assertTrue(
            expr=self.fixture.exists_by_value_and_type(
                value=self.TEST_USER_EMAIL, identifier_type=IdentifierType.EMAIL
            )  # assert Identifier Type works correctly
        )

    def test_create_empty_identifier_expect_exception(self):
        """Test creating an empty identifier and expecting an exception."""
        user_id = TestUtils.create_user().user_id
        email = ""

        with self.assertRaises(expected_exception=ServerException) as context:
            self.fixture.create(
                user_id=user_id,
                identifier_type=IdentifierType.EMAIL,
                identifier_value=email,
                status=IdentifierStatus.PRIMARY,
            )
        e: ServerException = context.exception
        self.assertIsNotNone(e)
        self.assertEqual(e.validation_messages[0], second="identifierValue: cannot be blank")

    def test_create_identifier_expect_success(self):
        """Test creating an identifier successfully."""
        user_id = TestUtils.create_user().user_id
        email: str = TestUtils.create_random_test_email()
        rsp: Identifier = self.fixture.create(
            user_id=user_id,
            identifier_type=IdentifierType.EMAIL,
            identifier_value=email,
            status=IdentifierStatus.PRIMARY,
        )
        self.assertEqual(user_id, rsp.user_id)
        self.assertEqual(email, rsp.value)
        self.assertEqual(IdentifierType.EMAIL, rsp.type)

    def test_get_identifiers_for_user_id_expect_list_of_identifiers(self):
        """Test case for searching identifiers by user ID."""
        ret: IdentifierList = self.fixture.list_identifiers(user_id=self.TEST_USER_ID)
        any(x.identifier_id == self.TEST_USER_EMAIL_IDENTIFIER.identifier_id for x in ret.identifiers)
        self.assertEqual(2, len(ret.identifiers))

    def test_list_identifiers_all_expect_success(self):
        """Test listing all identifiers with success."""
        ret: IdentifierList = self.fixture.list_identifiers(sort=None, page_size=100)
        self.assertIsNotNone(ret)

    def test_update_identifier_expect_success(self):
        """Test updating an identifier's status."""
        self.fixture.update_status(
            user_id=self.TEST_USER_EMAIL_IDENTIFIER.user_id,
            identifier_id=self.TEST_USER_EMAIL_IDENTIFIER.identifier_id,
            status=IdentifierStatus.PENDING,
        )
        ret: IdentifierList = self.fixture.list_identifiers(
            identifier_value=self.TEST_USER_EMAIL_IDENTIFIER.value, identifier_type=self.TEST_USER_EMAIL_IDENTIFIER.type
        )
        assert ret.identifiers[0].status == IdentifierStatus.PENDING

        self.fixture.update_status(
            user_id=self.TEST_USER_EMAIL_IDENTIFIER.user_id,
            identifier_id=self.TEST_USER_EMAIL_IDENTIFIER.identifier_id,
            status=IdentifierStatus.PRIMARY,
        )
        ret = self.fixture.list_identifiers(
            identifier_value=self.TEST_USER_EMAIL_IDENTIFIER.value, identifier_type=self.TEST_USER_EMAIL_IDENTIFIER.type
        )
        self.assertEqual(IdentifierStatus.PRIMARY, ret.identifiers[0].status)

    def test_list_all_emails_by_user_id_expect_success(self):
        """Test listing all emails by user ID with success."""
        test_size = 21
        for _ in range(test_size):
            self.fixture.create(
                user_id=self.TEST_USER_ID,
                identifier_type=IdentifierType.EMAIL,
                identifier_value=TestUtils.create_random_test_email(),
                status=IdentifierStatus.VERIFIED,
            )

        ret: List[Identifier] = self.fixture.list_all_emails_by_user_id(user_id=self.TEST_USER_ID)
        self.assertEqual(first=test_size + 1, second=len(ret))

    def test_delete_expect_success(self):
        identifier_created: Identifier = self.fixture.create(
            user_id=self.TEST_USER_ID,
            identifier_type=IdentifierType.EMAIL,
            identifier_value=TestUtils.create_random_test_email(),
            status=IdentifierStatus.VERIFIED,
        )
        self.fixture.delete(identifier_id=identifier_created.identifier_id, user_id=identifier_created.user_id)



import urllib.parse

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Annotated, List, Optional

from corbado_python_sdk.exceptions import ServerException
from corbado_python_sdk.generated import (
    ApiException,
    Identifier,
    IdentifierCreateReq,
    IdentifierList,
    IdentifiersApi,
    IdentifierStatus,
    IdentifierType,
    IdentifierUpdateReq,
)
from corbado_python_sdk.generated.models.paging import Paging
from corbado_python_sdk.services.interface.identifier_interface import (
    IdentifierInterface,
)


class IdentifierService(BaseModel, IdentifierInterface):
    """This class provides functionality for managing login identifiers."""

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )
    client: IdentifiersApi

    def create(
        self,
        user_id: Annotated[StrictStr, Field(description="ID of user")],
        identifier_type: IdentifierType,
        identifier_value: StrictStr,
        status: IdentifierStatus,
    ) -> Identifier:
        """Create a new identifier for the given user.

        Args:
            user_id (StrictStr): The ID of the user to whom the identifier will be assigned.
            identifier_type (IdentifierType): The type of identifier to create (e.g., email, phone).
            identifier_value (StrictStr): The value of the identifier (e.g., the email address or phone number).
            status (IdentifierStatus): The status of the identifier (e.g., active, inactive).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            Identifier: The created identifier object.
        """
        try:
            return self.client.identifier_create(
                user_id=user_id,
                identifier_create_req=IdentifierCreateReq(
                    identifierType=identifier_type, identifierValue=identifier_value, status=status
                ),
            )
        except ApiException as e:
            raise ServerException(e)

    def list_identifiers(
        self,
        sort: Optional[StrictStr] = None,
        filters: Optional[List[StrictStr]] = None,
        page: StrictInt = 1,
        page_size: StrictInt = 10,
        user_id: Optional[StrictStr] = None,
        identifier_type: Optional[IdentifierType] = None,
        identifier_value: Optional[StrictStr] = None,
    ) -> IdentifierList:
        """List identifiers with optional filters and pagination.

        Args:
            sort (Optional[StrictStr], optional): The sorting order for the results (e.g., 'createdAt:desc'). Defaults to None.
            filters (Optional[List[StrictStr]], optional): A list of filters to
                apply to the results (e.g., 'status:eq:active'). Defaults to None.
            page (StrictInt): The page number for pagination. Defaults to 1.
            page_size (StrictInt): The number of results per page. Defaults to 10.
            user_id (Optional[StrictStr], optional): Filter results by user ID. Defaults to None.
            identifier_type (Optional[IdentifierType], optional): Filter results by identifier type. Defaults to None.
            identifier_value (Optional[StrictStr], optional): Filter results by identifier value. Defaults to None.

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            IdentifierList: A list of identifiers that match the criteria.
        """
        if user_id:
            filters = filters or []
            if user_id.startswith("usr-"):
                user_id = user_id[4:]
            filters.append(f"userID:eq:{user_id}")
        if identifier_type:
            filters = filters or []
            encoded_type: StrictStr = urllib.parse.quote_plus(identifier_type)
            filters.append(f"identifierType:eq:{encoded_type}")
        if identifier_value:
            encoded_value: StrictStr = urllib.parse.quote_plus(identifier_value)
            filters = filters or []
            filters.append(f"identifierValue:eq:{encoded_value}")
        try:
            return self.client.identifier_list(sort=sort, filter=filters, page=page, page_size=page_size)
        except ApiException as e:
            raise ServerException(e)

    def list_all_emails_by_user_id(
        self,
        user_id: Annotated[StrictStr, Field(description="The user ID")],
    ) -> List[Identifier]:
        """Retrieve all email identifiers for a specific user, handling pagination.

        Args:
            user_id (StrictStr): The ID of the user whose email identifiers are to be listed.

        Returns:
            List[Identifier]: A list of email identifiers associated with the user.
        """
        identifiers: List[Identifier] = []
        first_res: IdentifierList = self.list_identifiers(user_id=user_id, identifier_type=IdentifierType.EMAIL)
        identifiers.extend(first_res.identifiers)

        paging: Paging = first_res.paging
        while paging.page < paging.total_pages:
            paging.page += 1
            temp_res: IdentifierList = self.list_identifiers(
                user_id=user_id, identifier_type=IdentifierType.EMAIL, page=paging.page
            )
            identifiers.extend(temp_res.identifiers)

        return identifiers

    def exists_by_value_and_type(
        self,
        value: Annotated[StrictStr, Field(description="The identifier value")],
        identifier_type: IdentifierType,
    ) -> bool:
        """Check if an identifier with a specific value and type exists.

        Args:
            value (StrictStr): The value of the identifier to check (e.g., an email address or phone number).
            identifier_type (IdentifierType): The type of identifier to check (e.g., email, phone).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            bool: True if the identifier exists, False otherwise.
        """
        try:
            ret: IdentifierList = self.list_identifiers(identifier_value=value, identifier_type=identifier_type)
            return bool(ret.identifiers)
        except ApiException as e:
            raise ServerException(e)

    def update_status(
        self,
        user_id: Annotated[StrictStr, Field(description="ID of user")],
        identifier_id: Annotated[StrictStr, Field(description="ID of login identifier")],
        status: IdentifierStatus,
    ) -> Identifier:
        """Update the status of a specific identifier.

        Args:
            user_id (StrictStr): The ID of the user to whom the identifier belongs.
            identifier_id (StrictStr): The ID of the identifier to update.
            status (IdentifierStatus): The new status for the identifier (e.g., active, inactive).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            Identifier: The updated identifier object.
        """
        try:
            identifier_update_req = IdentifierUpdateReq(status=status)
            return self.client.identifier_update(
                user_id=user_id, identifier_id=identifier_id, identifier_update_req=identifier_update_req
            )
        except ApiException as e:
            raise ServerException(e)

    def delete(
        self,
        user_id: StrictStr,
        identifier_id: StrictStr,
    ) -> None:
        """Delete the identifier.

        Args:
            user_id (StrictStr): ID of user.
            identifier_id (StrictStr): ID of login identifier.

        Raises:
            ServerException: _description_
        """
        try:
            self.client.identifier_delete(user_id=user_id, identifier_id=identifier_id)
        except ApiException as e:
            raise ServerException(e)



from .identifier_interface import IdentifierInterface
from .session_interface import SessionInterface
from .user_interface import UserInterface

__all__ = [
    "IdentifierInterface",
    "UserInterface",
    "SessionInterface",
]



from math import e

from pydantic import ConfigDict

from corbado_python_sdk.generated.models.user import User


class UserEntity(User):
    """Represents a user entity."""

    model_config = ConfigDict(populate_by_name=True, from_attributes=True)

    @classmethod
    def from_user(cls, user: User) -> "UserEntity":
        """Create a UserEntity instance from a User.

        Args:
            user (User): User object

        Returns:
            UserEntity: UserEntity
        """
        return UserEntity(
            userID=user.user_id,
            status=user.status,
            explicitWebauthnID=user.explicit_webauthn_id,
            fullName=user.full_name,
        )



# type: ignore
import os
import unittest
from time import time
from unittest.mock import AsyncMock, MagicMock, patch

from jwt import encode
from pydantic import ValidationError

from corbado_python_sdk.entities.user_entity import UserEntity
from corbado_python_sdk.services.implementation import SessionService

TEST_NAME = "Test Name"
TEST_EMAIL = "test@email.com"
TEST_PHONE_NUMBER = "+012345678"
TEST_USER_ID = "12345"


class TestBase(unittest.TestCase):
    """Base class for all test using mocking for SessionService.

    Args:
        unittest (_type_): unittest

    Raises:
        FileNotFoundError: If no private key was found

    """

    mock_urlopen = None

    @classmethod
    def setUpClass(cls) -> None:
        # Setup, read private key and jwks for mocking
        cls.session_service: SessionService = cls.create_session_service()

        jwks_path: str = os.path.join(os.path.dirname(__file__), "test_data", "jwks.json")
        try:
            with open(file=jwks_path, mode="rb") as private_key_file:
                cls.jwks: bytes = private_key_file.read()
        except FileNotFoundError:
            raise FileNotFoundError("Failed to read private key file")

    def setUp(self) -> None:
        self.my_patch = patch("urllib.request.urlopen")
        self.mock_urlopen: MagicMock | AsyncMock = self.my_patch.start()
        self.addCleanup(self.my_patch.stop)
        # Mock the response
        self.mock_response = MagicMock()
        self.mock_response.__enter__.return_value = self.mock_response
        self.mock_response.read.return_value = self.jwks
        self.mock_urlopen.return_value = self.mock_response

    @classmethod
    def create_session_service(cls) -> SessionService:
        """Create test configuration of SessionService.

        Warning! You should normally use SessionInterface from CorbadoSDK for non-test purposes.

        Returns:
            SessionService: SessionService instance
        """
        return SessionService(
            issuer="https://auth.acme.com",
            jwks_uri="https://example_uri.com",  # does not matter, url access is mocked
            short_session_cookie_name="cbo_short_session",
        )

    def tearDown(self) -> None:
        self.my_patch.stop()

    def _provide_jwts(self):
        """Provide list of jwts with expected test results."""
        return [
            # JWT with invalid format
            (False, "invalid"),
            # JWT signed with wrong algorithm (HS256 instead of RS256)
            (
                False,
                """eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6
                IkpvaG4gRG9lIiwiYWRtaW4iOnRydWV9.dyt0CoTl4WoVjAHI9Q_CwSKhl6d_9rhM3NrXuJttkao""",
            ),
            # Not before (nfb) in future
            (False, self._generate_jwt(iss="https://auth.acme.com", exp=int(time()) + 100, nbf=int(time()) + 100)),
            # Expired (exp)
            (False, self._generate_jwt(iss="https://auth.acme.com", exp=int(time()) - 100, nbf=int(time()) - 100)),
            # Invalid issuer (iss)
            (False, self._generate_jwt(iss="https://invalid.com", exp=int(time()) + 100, nbf=int(time()) - 100)),
            # Success
            (True, self._generate_jwt(iss="https://auth.acme.com", exp=int(time()) + 100, nbf=int(time()) - 100)),
        ]

    @classmethod
    def _generate_jwt(cls, iss: str, exp: int, nbf: int) -> str:
        payload = {
            "iss": iss,
            "iat": int(time()),
            "exp": exp,
            "nbf": nbf,
            "sub": TEST_USER_ID,
            "name": TEST_NAME,
        }

        private_key_path: str = os.path.join(os.path.dirname(__file__), "test_data", "privateKey.pem")
        try:
            with open(file=private_key_path, mode="rb") as private_key_file:
                private_key = private_key_file.read()
        except FileNotFoundError:
            raise FileNotFoundError("Failed to read private key file")

        return encode(payload, private_key, algorithm="RS256", headers={"kid": "kid123"})


class TestSessionService(TestBase):
    def test_get_and_validate_short_session_value(self):
        for valid, token in self._provide_jwts():
            result: UserEntity = self.session_service.get_and_validate_short_session_value(short_session=token)

            self.assertEqual(valid, result.authenticated)

            if valid:
                self.assertEqual(first=TEST_NAME, second=result.full_name)
                self.assertEqual(TEST_USER_ID, result.user_id)

    def test_cache_jwk_set_used_expect_reduced_urlopen_calls(self):
        jwt: str = self._generate_jwt(iss="https://auth.acme.com", exp=int(time()) + 100, nbf=int(time()) - 100)
        self.session_service.get_and_validate_short_session_value(short_session=jwt)
        num_calls: int = self.mock_urlopen.call_count
        for _i in range(3):
            self.session_service.get_and_validate_short_session_value(short_session=jwt)
        self.assertEqual(num_calls, self.mock_urlopen.call_count)

    def test_generate_jwt(self):
        iss = "issuer"
        exp = 1234567890  # Example expiration time
        nbf = 0  # Example not-before time

        jwt_token: str = self._generate_jwt(iss=iss, exp=exp, nbf=nbf)
        self.assertIsNotNone(jwt_token)

    def test_init_parameters(self):
        test_cases = [
            # Valid session service
            ({"issuer": "s", "jwks_uri": "2", "short_session_cookie_name": "name"}, True),
            # Test empty issuer
            ({"issuer": "", "jwks_uri": "2", "short_session_cookie_name": "name"}, False),
            # Test empty jwks_uri
            ({"issuer": "s", "jwks_uri": "", "short_session_cookie_name": "name"}, False),
            # Tesft empty short_session_cookie_name
            ({"issuer": "s", "jwks_uri": "2", "short_session_cookie_name": ""}, False),
        ]

        for params, expected_result in test_cases:
            if expected_result:
                # No exception should be raised
                assert isinstance(SessionService(**params), SessionService)
            else:
                # ValidationError should be raised
                with self.assertRaises(ValidationError):
                    SessionService(**params)



# coding: utf-8

"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AppType(str, Enum):
    """
    Application type
    """

    """
    allowed enum values
    """
    EMPTY = 'empty'
    WEB = 'web'
    NATIVE = 'native'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AppType from a JSON string"""
        return cls(json.loads(json_str))





from abc import ABC, abstractmethod

from pydantic import Field, StrictInt, StrictStr
from typing_extensions import Annotated, List, Optional

from corbado_python_sdk.generated import (
    Identifier,
    IdentifierList,
    IdentifierStatus,
    IdentifierType,
)


class IdentifierInterface(ABC):
    """This class provides functionality for managing login identifiers."""

    @abstractmethod
    def create(
        self,
        user_id: Annotated[StrictStr, Field(description="ID of user")],
        identifier_type: IdentifierType,
        identifier_value: StrictStr,
        status: IdentifierStatus,
    ) -> Identifier:
        """Create a new identifier for the given user.

        Args:
            user_id (StrictStr): The ID of the user to whom the identifier will be assigned.
            identifier_type (IdentifierType): The type of identifier to create (e.g., email, phone).
            identifier_value (StrictStr): The value of the identifier (e.g., the email address or phone number).
            status (IdentifierStatus): The status of the identifier (e.g., active, inactive).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            Identifier: The created identifier object.
        """
        pass

    @abstractmethod
    def list_identifiers(
        self,
        sort: Optional[StrictStr] = None,
        filters: Optional[List[StrictStr]] = None,
        page: StrictInt = 1,
        page_size: StrictInt = 10,
        user_id: Optional[StrictStr] = None,
        identifier_type: Optional[IdentifierType] = None,
        identifier_value: Optional[StrictStr] = None,
    ) -> IdentifierList:
        """List identifiers with optional filters and pagination.

        Args:
            sort (Optional[StrictStr], optional): The sorting order for the results (e.g., 'createdAt:desc'). Defaults to None.
            filters (Optional[List[StrictStr]], optional): A list of filters to
                apply to the results (e.g., 'status:eq:active'). Defaults to None.
            page (StrictInt): The page number for pagination. Defaults to 1.
            page_size (StrictInt): The number of results per page. Defaults to 10.
            user_id (Optional[StrictStr], optional): Filter results by user ID. Defaults to None.
            identifier_type (Optional[IdentifierType], optional): Filter results by identifier type. Defaults to None.
            identifier_value (Optional[StrictStr], optional): Filter results by identifier value. Defaults to None.

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            IdentifierList: A list of identifiers that match the criteria.
        """
        pass

    @abstractmethod
    def list_all_emails_by_user_id(
        self,
        user_id: Annotated[StrictStr, Field(description="The user ID")],
    ) -> List[Identifier]:
        """Retrieve all email identifiers for a specific user, handling pagination.

        Args:
            user_id (StrictStr): The ID of the user whose email identifiers are to be listed.

        Returns:
            List[Identifier]: A list of email identifiers associated with the user.
        """
        pass

    @abstractmethod
    def exists_by_value_and_type(
        self,
        value: Annotated[StrictStr, Field(description="The identifier value")],
        identifier_type: IdentifierType,
    ) -> bool:
        """Check if an identifier with a specific value and type exists.

        Args:
            value (StrictStr): The value of the identifier to check (e.g., an email address or phone number).
            identifier_type (IdentifierType): The type of identifier to check (e.g., email, phone).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            bool: True if the identifier exists, False otherwise.
        """
        pass

    @abstractmethod
    def update_status(
        self,
        user_id: Annotated[StrictStr, Field(description="ID of user")],
        identifier_id: Annotated[StrictStr, Field(description="ID of login identifier")],
        status: IdentifierStatus,
    ) -> Identifier:
        """Update the status of a specific identifier.

        Args:
            user_id (StrictStr): The ID of the user to whom the identifier belongs.
            identifier_id (StrictStr): The ID of the identifier to update.
            status (IdentifierStatus): The new status for the identifier (e.g., active, inactive).

        Raises:
            ServerException: If there is an error from the API server.

        Returns:
            Identifier: The updated identifier object.
        """
        pass

    @abstractmethod
    def delete(
        self,
        user_id: StrictStr,
        identifier_id: StrictStr,
    ):
        """Delete the identifier.

        Args:
            user_id (StrictStr): ID of user.
            identifier_id (StrictStr): ID of login identifier.

        Raises:
            ServerException: _description_

        """
        pass



from .config import Config as Config
from .corbado_sdk import CorbadoSDK as CorbadoSDK
from .entities import UserEntity as UserEntity
from .exceptions import StandardException as StandardException
from .generated import (
    Identifier,
    IdentifierCreateReq,
    IdentifierStatus,
    IdentifierType,
    UserCreateReq,
    UserStatus,
    UserUpdateReq,
)
from .services import IdentifierInterface, SessionInterface, UserInterface

__all__ = [
    "IdentifierCreateReq",
    "Identifier",
    "IdentifierStatus",
    "IdentifierType",
    "UserCreateReq",
    "UserUpdateReq",
    "IdentifierInterface",
    "UserStatus",
    "StandardException",
    "UserEntity",
    "CorbadoSDK",
    "Config",
    "UserInterface",
    "SessionInterface",
]



from pydantic import BaseModel, Field
from typing_extensions import Optional


class SessionValidationResult(BaseModel):
    """Result class for SessionService validation."""

    authenticated: bool = Field(default=False, description="Indicates success of validation by session service.")
    user_id: Optional[str] = Field(default=None, description="The user ID.")
    full_name: Optional[str] = Field(default=None, description="The full name.")



import base64
import json
import platform
from importlib.metadata import version

from pydantic import BaseModel, ConfigDict
from typing_extensions import Dict, Optional

from corbado_python_sdk import Config
from corbado_python_sdk.generated.api import IdentifiersApi, UsersApi
from corbado_python_sdk.generated.api_client import ApiClient
from corbado_python_sdk.generated.configuration import Configuration
from corbado_python_sdk.services.implementation import (
    IdentifierService,
    SessionService,
    UserService,
)
from corbado_python_sdk.services.interface import (
    IdentifierInterface,
    SessionInterface,
    UserInterface,
)

CORBADO_HEADER_NAME = "X-Corbado-SDK"


class CorbadoSDK(BaseModel):
    """
    Entry point for the Corbado SDK.

    This class provides various interfaces to interact with the Corbado API,
    including user management, session handling, validation, and OTP services.

    Attributes:
        config (Config): The configuration object for the SDK.
        api_client (ApiClient): The API client used to make requests to the backend API.
        sessions (SessionService): The session service service.
        users (UserInterface): The user service.
    """

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )
    config: Config
    _api_client: Optional[ApiClient] = None
    _sessions: Optional[SessionService] = None
    _users: Optional[UserInterface] = None
    _identifiers: Optional[IdentifierInterface] = None

    @property
    def api_client(self) -> ApiClient:
        """Get ApiClient.

        Returns:
            ApiClient: ApiClient object.
        """
        if not self._api_client:
            self._api_client = ApiClient(configuration=self._create_generated_configuration())
            python_version: str = platform.python_version()
            data: Dict[str, str] = {
                "name": "Python SDK",
                "sdkVersion": version(distribution_name="passkeys"),
                "languageVersion": python_version,
            }
            self._api_client.set_default_header(header_name=CORBADO_HEADER_NAME, header_value=json.dumps(data))  # type: ignore
        return self._api_client

    # --------- Interfaces ---------------#
    @property
    def sessions(self) -> SessionInterface:
        """Get user SessionInterface.

        Returns:
            SessionInterface: SessionService object.
        """
        if not self._sessions:
            self._sessions = SessionService(
                short_session_cookie_name=self.config.short_session_cookie_name,
                issuer=self.config.issuer,
                jwks_uri=self.config.frontend_api + "/.well-known/jwks",
            )

        return self._sessions

    @property
    def users(self) -> UserInterface:
        """Get user service.

        Returns:
            UserInterface: UserService object.
        """
        if not self._users:
            client = UsersApi(api_client=self.api_client)
            self._users = UserService(client=client)

        return self._users

    @property
    def identifiers(self) -> IdentifierInterface:
        """Get identifier service.

        Returns:
            IdentifierInterface: IdentifierService object.
        """
        if not self._identifiers:
            client = IdentifiersApi(api_client=self.api_client)
            self._identifiers = IdentifierService(client=client)

        return self._identifiers

    # ----------- Functions ----------#

    def _create_generated_configuration(self) -> Configuration:
        """Create configuration (generated class).

        Returns:
            Configuration: Configuration object.
        """
        return Configuration(
            host=self.config.backend_api,
            username=self.config.project_id,
            password=self.config.api_secret,
            access_token=None,
            api_key={"projectID": self.config.project_id},
        )

    def _generate_basic_auth_header(self, username: str, password: str) -> str:
        """Generate basic auth header.

        Args:
            username (str): ProjectId
            password (str): API Secret

        Returns:
            str: base64 encoded header value for basic authentication
        """
        credentials: str = f"{username}:{password}"
        encoded_credentials: str = base64.b64encode(credentials.encode()).decode()
        return f"Basic {encoded_credentials}"



import os
import random
import string

from typing_extensions import LiteralString

from corbado_python_sdk import CorbadoSDK, UserStatus
from corbado_python_sdk.config import Config
from corbado_python_sdk.entities import UserEntity
from corbado_python_sdk.generated import UserCreateReq


class TestUtils:
    """Utility class for test."""

    CORBADO_API_SECRET: str = "CORBADO_API_SECRET"
    CORBADO_PROJECT_ID: str = "CORBADO_PROJECT_ID"
    CORBADO_BACKEND_API: str = "CORBADO_BACKEND_API"

    @classmethod
    def instantiate_sdk(cls) -> CorbadoSDK:
        """Instantiate SDK with parameters from environment variables."""
        config: Config = Config(
            api_secret=os.getenv(key=TestUtils.CORBADO_API_SECRET, default="missing CORBADO_API_SECRET"),
            project_id=os.getenv(key=TestUtils.CORBADO_PROJECT_ID, default="missing CORBADO_PROJECT_ID"),
            backend_api=os.getenv(key=TestUtils.CORBADO_BACKEND_API, default="https://backendapi.cloud.corbado.io/v2"),
        )
        return CorbadoSDK(config=config)

    @classmethod
    def generate_string(cls, length: int, characters: str) -> str:
        """Generate a random string of specified length using the provided characters."""
        return "".join(random.choice(characters) for _ in range(length))

    @classmethod
    def create_random_test_name(cls) -> str:
        """Generate a random test name."""
        characters: LiteralString = string.ascii_letters + string.digits
        return TestUtils.generate_string(10, characters)

    @classmethod
    def create_random_test_email(cls) -> str:
        """Generate a random test email."""
        characters: LiteralString = string.ascii_letters + string.digits
        random_string = TestUtils.generate_string(10, characters)
        return f"integration-test+{random_string}@corbado.com"

    @classmethod
    def create_random_test_phone_number(cls) -> str:
        """Generate a random test phone number."""
        return "+491509" + TestUtils.generate_string(7, string.digits)

    @classmethod
    def create_user(cls) -> UserEntity:
        """Create a user and return the user ID."""
        req = UserCreateReq(fullName=TestUtils.create_random_test_name(), status=UserStatus.ACTIVE)
        rsp: UserEntity = TestUtils.instantiate_sdk().users.create_from_request(request=req)
        return rsp



# coding: utf-8

"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class PasskeyEventType(str, Enum):
    """
    PasskeyEventType
    """

    """
    allowed enum values
    """
    USER_MINUS_LOGIN_MINUS_BLACKLISTED = 'user-login-blacklisted'
    LOGIN_MINUS_EXPLICIT_MINUS_ABORT = 'login-explicit-abort'
    LOGIN_MINUS_ERROR = 'login-error'
    LOGIN_MINUS_ONE_MINUS_TAP_MINUS_SWITCH = 'login-one-tap-switch'
    USER_MINUS_APPEND_MINUS_AFTER_MINUS_CROSS_MINUS_PLATFORM_MINUS_BLACKLISTED = 'user-append-after-cross-platform-blacklisted'
    USER_MINUS_APPEND_MINUS_AFTER_MINUS_LOGIN_MINUS_ERROR_MINUS_BLACKLISTED = 'user-append-after-login-error-blacklisted'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PasskeyEventType from a JSON string"""
        return cls(json.loads(json_str))





from urllib import parse


def url_validator(url: str) -> str:
    """Url validator.

    Args:
        url (str): URL.

    Raises:
        ValueError: Raises ValueError if url is not valid.

    Returns:
        str: validated url.
    """
    try:
        parts: parse.ParseResult = parse.urlparse(url)
    except ValueError:
        raise ValueError("Assert failed: parse_url() returned error")

    if parts.scheme != "https":
        raise ValueError("Assert failed: scheme needs to be https")

    if not parts.hostname:
        raise ValueError("Assert failed: host is empty")

    if parts.username:
        raise ValueError("Assert failed: username needs to be empty")

    if parts.password:
        raise ValueError("Assert failed: password needs to be empty")

    if parts.query:
        raise ValueError("Assert failed: querystring needs to be empty")

    if parts.fragment:
        raise ValueError("Assert failed: fragment needs to be empty")
    return url



# coding: utf-8


import os

from setuptools import find_namespace_packages, setup

base_dir: str = os.path.abspath(os.path.dirname(__file__))


with open(os.path.join(base_dir, "VERSION")) as version_file:
    version: str = version_file.read().strip()

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.12",
    "Operating System :: OS Independent",
    "Topic :: Software Development :: Libraries :: Python Modules",
]


NAME = "passkeys"
VERSION: str = version
PYTHON_REQUIRES = ">=3.8"

INSTALL_REQUIRES = [
    "urllib3 >= 1.25.3",
    "python-dateutil",  # used in generated code
    "pydantic >= 2",
    "typing-extensions >= 4.7.1",
    "pyopenssl",
    "PyJWT",
]

setup(
    name=NAME,
    version=VERSION,
    description="Corbado SDK",
    author="Corbado team",
    author_email="support@corbado.com",
    url="https://github.com/corbado/corbado-python",
    keywords=["Corbado", "Corbado Python SDK"],
    install_requires=INSTALL_REQUIRES,
    packages=find_namespace_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    classifiers=CLASSIFIERS,
    long_description_content_type="text/markdown",
    long_description=open(os.path.join(base_dir, "README.md"), encoding="utf-8").read(),
    package_data={"corbado_sdk": ["py.typed"]},
)



import json
from typing import Any, Dict, List

from corbado_python_sdk.exceptions.standard_exception import StandardException
from corbado_python_sdk.generated import GenericRsp, RequestData


class Util:
    """Helper class containing various utility methods."""

    @staticmethod
    def json_encode(data: Any) -> str:
        """JSON encode.

        Args:
            data (Any): Data to be encoded.

        Raises:
            StandardException: If encoding fails.

        Returns:
            str: Encoded JSON string.
        """
        try:
            json_str: str = json.dumps(data)
        except TypeError as e:
            raise StandardException(f"json.dumps() failed: {e}")
        except OverflowError as e:
            raise StandardException(f"json.dumps() failed: {e}")
        except ValueError as e:
            raise StandardException(f"json.dumps() failed: {e}")
        except RecursionError as e:
            raise StandardException(f"json.dumps() failed: {e}")
        except Exception as e:
            raise StandardException(f"json.dumps() failed: {e}")
        return json_str

    @staticmethod
    def json_decode(data: str) -> Dict[str, Any]:
        """
        JSON decode.

        Args:
            data (str): Data to be decoded.

        Raises:
            StandardException: If decoding fails.

        Returns:
            Dict[str, Any]: Decoded data as a dictionary.
        """
        try:
            decoded_data: Dict[str, Any] = json.loads(data)
        except json.JSONDecodeError as e:
            raise StandardException(f"json.loads() failed: {e}")
        except TypeError as e:
            raise StandardException(f"json.loads() failed: {e}")
        except Exception as e:
            raise StandardException(f"json.loads() failed: {e}")

        return decoded_data

    @staticmethod
    def is_error_http_status_code(status_code: int) -> bool:
        """
        Check if the status code indicates an error.

        Args:
            status_code (int): Status code.

        Returns:
            bool: True if status code indicates an error, False otherwise.
        """
        return status_code >= 300

    @staticmethod
    def hydrate_request_data(data: Dict[str, Any]) -> RequestData:
        """
        Hydrate RequestData object from dictionary.

        Args:
            data (Dict[str, Any]): Data.

        Raises:
            StandardException: If required keys are missing in the data.

        Returns:
            RequestData: RequestData object.
        """
        keys_to_check: List[str] = ["requestID", "link"]
        for key in keys_to_check:
            if key not in data:
                raise StandardException(f"Key '{key}' not found in data")
        return RequestData(requestID=data["requestID"], link=data["link"])

    @staticmethod
    def hydrate_response(data: Dict[str, Any]) -> GenericRsp:
        """
        Hydrate GenericRsp object from dictionary.

        Args:
            data (Dict[str, Any]): Data.

        Raises:
            StandardException: If required keys are missing in the data.

        Returns:
            GenericRsp: GenericRsp object.
        """
        keys_to_check: List[str] = ["httpStatusCode", "message", "requestData", "runtime"]
        for key in keys_to_check:
            if key not in data:
                raise StandardException(f"Key '{key}' not found in data")
        request_data: RequestData = Util.hydrate_request_data(data["requestData"])
        return GenericRsp(
            httpStatusCode=data["httpStatusCode"],
            message=data["message"],
            requestData=request_data,
            runtime=data["runtime"],
        )



from typing import Any, Dict, List, Optional

from corbado_python_sdk.exceptions.standard_exception import StandardException
from corbado_python_sdk.generated import ApiException

from ..utils.util import Util


class ServerException(Exception):
    """Custom exception class for server-related errors."""

    def __init__(self, e: ApiException) -> None:
        """Convert ApiException to ServerException.

        Args:
            e (ApiException): Exception to be converted.

        Raises:
            StandardException: If response body is not a string.

        """
        __body = e.body  # type: ignore
        if not isinstance(__body, str):
            raise StandardException("Response body is not a string")
        __data: Dict[str, Any] = Util.json_decode(__body)
        self.__initialize(
            http_status_code=__data["httpStatusCode"],
            message=__data["message"],
            request_data=__data["requestData"],
            runtime=__data["runtime"],
            error=__data["error"],
        )

    def __initialize(
        self,
        http_status_code: int,
        message: str,
        request_data: Optional[Dict[str, Any]] = None,
        runtime: Optional[float] = None,
        error: Optional[Dict[str, Any]] = None,
    ) -> None:
        """Initialize ServerException with specific attributes.

        Args:
            http_status_code (int):  HTTP status code associated with the error
            message (str): Error message
            request_data (Optional[Dict[str, Any]], optional): Additional data associated with the request (optional)
            runtime (Optional[float], optional): Runtime information (optional). Defaults to None.
            error (Optional[Dict[str, Any]], optional): Specific error information (optional). Defaults to None.
        """
        self.http_status_code: int = http_status_code
        self.request_data: Dict[str, Any] = request_data or {}
        self.runtime: Optional[float] = runtime
        self.error: Dict[str, Any] = error or {}
        self.error_type: str = self.error.get("type", "")

        self.validation_messages: List[str] = self._get_validation_messages()
        message += (
            f' (HTTP status code: {http_status_code}, validation messages: {"; ".join(self.validation_messages)})'
        )

        super().__init__(message)

    def _get_validation_messages(self) -> List[str]:
        """Get the validation messages from the error information.

        Returns:
            List[str]: List of validation messages
        """
        validation: List[Dict[str, Any]] = self.error.get("validation", [])
        return [f"{item['field']}: {item['message']}" for item in validation]

    @property
    def request_id(self) -> str:
        """Get request ID.

        Returns:
            str: Request ID.
        """
        return self.request_data.get("requestID") or ""



import unittest

from corbado_python_sdk import CorbadoSDK, UserEntity, UserStatus
from corbado_python_sdk.exceptions import ServerException
from corbado_python_sdk.services.interface.user_interface import UserInterface
from tests.utils import TestUtils


class TestBase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.sdk: CorbadoSDK = TestUtils.instantiate_sdk()
        cls.fixture: UserInterface = cls.sdk.users


class TestMisc(TestBase):

    def test_instantiate_sdk_expect_not_none(self) -> None:
        sdk: CorbadoSDK = self.sdk
        self.assertIsNotNone(obj=sdk)


class UserCreateTest(TestBase):
    """Test cases for user creation."""

    def test_user_create_expect_success(self) -> None:
        """Test case for successful user creation."""
        test_status = UserStatus.ACTIVE
        test_name: str = TestUtils.create_random_test_name()
        rsp: UserEntity = self.fixture.create(status=test_status, full_name=test_name)

        self.assertEqual(first=test_status, second=rsp.status)
        self.assertEqual(first=test_name, second=rsp.full_name)


class TestUserDelete(TestBase):
    """Tests for the user deletion functionality."""

    def test_user_delete_expect_not_found(self) -> None:
        """Test for deleting a user that does not exist."""
        with self.assertRaises(expected_exception=ServerException) as context:
            self.fixture.delete(user_id="usr-123456789")

        e: ServerException = context.exception
        self.assertIsNotNone(e)
        self.assertEqual(400, e.http_status_code)
        self.assertListEqual(["userID: does not exist"], e.validation_messages)

    def test_user_delete_expect_success(self) -> None:
        """Test for successfully deleting a user."""
        user: UserEntity = TestUtils.create_user()

        self.fixture.delete(user_id=user.user_id)


class TestUserGet(TestBase):
    """Tests for the user retrieval functionality."""

    def test_user_get_expect_not_found(self) -> None:
        """Test for retrieving a user that does not exist."""
        with self.assertRaises(expected_exception=ServerException) as context:
            self.fixture.get(user_id="usr-123456789")

        e: ServerException = context.exception
        self.assertIsNotNone(e)
        self.assertEqual(400, e.http_status_code)

    def test_user_get_expect_success(self) -> None:
        """Test for successfully retrieving a user."""
        user: UserEntity = TestUtils.create_user()

        rsp: UserEntity = self.fixture.get(user_id=user.user_id)
        self.assertEqual(first=user, second=rsp)



import unittest

from corbado_python_sdk import Config


class TestConfig(unittest.TestCase):

    def test_set_frontend_api(self):

        for frontend_api, valid in self.provide_urls():
            try:
                config = Config(project_id="pro-123", api_secret="corbado1_123")
                config.frontend_api = frontend_api
                error = False
            except ValueError:
                error = True
            self.assertEqual(valid, not error)

    def test_set_backend_api(self):
        for backend_api, valid in self.provide_urls():
            try:
                config = Config(project_id="pro-123", api_secret="corbado1_123")
                config.backend_api = backend_api
                error = False
            except ValueError:
                error = True
            self.assertEqual(valid, second=not error)

    def test_get_frontend_api(self):
        config = Config(project_id="pro-123", api_secret="corbado1_123")
        self.assertEqual("https://pro-123.frontendapi.corbado.io", config.frontend_api)

    def provide_urls(self):
        return [
            ("", False),
            ("xxx", False),
            ("http://auth.acme.com", False),  # Only HTTPS
            ("https://user@auth.acme.com", False),  # No user
            ("https://user:pass@auth.acme.com", False),  # No user no password
            ("https://auth.acme.com?xxx", False),  # No query string
            ("https://auth.acme.com#xxx", False),  # No fragment
            ("https://auth.acme.com", True),
        ]


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    Corbado Backend API

     # Introduction This documentation gives an overview of all Corbado Backend API calls to implement passwordless authentication with Passkeys. 

    The version of the OpenAPI document: 2.0.0
    Contact: support@corbado.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class DecisionTag(str, Enum):
    """
    DecisionTag
    """

    """
    allowed enum values
    """
    ENV_MINUS_NO_MINUS_PK_MINUS_SUPPORT = 'env-no-pk-support'
    USER_MINUS_NO_MINUS_PKS = 'user-no-pks'
    USER_MINUS_LOGIN_MINUS_BLACKLISTED = 'user-login-blacklisted'
    USER_MINUS_SECURITY_MINUS_KEY = 'user-security-key'
    USER_MINUS_POSITIVE_MINUS_ENV_MINUS_HISTORY = 'user-positive-env-history'
    USER_MINUS_NEGATIVE_MINUS_ENV_MINUS_HISTORY = 'user-negative-env-history'
    ENV_MINUS_BLACKLISTED = 'env-blacklisted'
    USER_MINUS_PLATFORM_MINUS_PK_MINUS_HIGH_MINUS_CONFIDENCE = 'user-platform-pk-high-confidence'
    USER_MINUS_CROSS_MINUS_PLATFORM_MINUS_PK_MINUS_HIGH_MINUS_CONFIDENCE = 'user-cross-platform-pk-high-confidence'
    USER_MINUS_ENV_MINUS_NO_MINUS_PKS = 'user-env-no-pks'
    DEFAULT_MINUS_DENY = 'default-deny'
    PASSKEY_MINUS_LIST_MINUS_INITIATED_MINUS_PROCESS = 'passkey-list-initiated-process'
    USER_MINUS_APPEND_MINUS_BLACKLISTED = 'user-append-blacklisted'
    PROCESS_MINUS_PK_MINUS_LOGIN_MINUS_SK_MINUS_COMPLETED = 'process-pk-login-sk-completed'
    PROCESS_MINUS_PK_MINUS_LOGIN_MINUS_PLATFORM_MINUS_COMPLETED = 'process-pk-login-platform-completed'
    PROCESS_MINUS_PK_MINUS_LOGIN_MINUS_NOT_MINUS_OFFERED = 'process-pk-login-not-offered'
    PROCESS_MINUS_PK_MINUS_LOGIN_MINUS_INCOMPLETE = 'process-pk-login-incomplete'
    PROCESS_MINUS_PK_MINUS_LOGIN_MINUS_CROSS_MINUS_PLATFORM_MINUS_COMPLETED = 'process-pk-login-cross-platform-completed'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of DecisionTag from a JSON string"""
        return cls(json.loads(json_str))


